"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.run = void 0;
const lodash_merge_1 = __importDefault(require("lodash.merge"));
const config_1 = require("../config");
const package_1 = require("../package");
const log_1 = require("../util/log");
function run() {
    return __awaiter(this, void 0, void 0, function* () {
        const config = yield (0, config_1.readConfig)();
        const { defaults, packages } = (0, config_1.parseConfig)(config);
        if (packages.length < 1) {
            log_1.log.warn `No packages could be found for this configuration.`;
            return;
        }
        const promises = packages.map((pkg) => {
            if (Array.isArray(pkg)) {
                const [name, config] = pkg;
                return (0, package_1.processPackage)(name, config.merge ? (0, lodash_merge_1.default)(defaults, config) : config);
            }
            return (0, package_1.processPackage)(pkg, defaults);
        });
        const results = yield Promise.all(promises);
        log_1.log.info `Generated types for ${results.filter((r) => !!r).length + ''} package(s) out of ${packages.length}.`;
    });
}
exports.run = run;
//# sourceMappingURL=run.js.map