"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.emitTypes = void 0;
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const log_1 = require("../util/log");
const shell_1 = require("../util/shell");
const info_1 = require("./info");
/**
 * Emit the declaration types for a package
 *
 * @param name The package name
 * @param cwd The package directory
 * @param info The package info
 * @returns True if the declaration types were emitted
 */
function emitTypes(name, cwd, info) {
    return __awaiter(this, void 0, void 0, function* () {
        const infoPath = path_1.default.join(cwd, info_1.filename);
        const [stdout, error] = yield (0, shell_1.childExec)(`tsc -p ${infoPath}`, { cwd });
        if (error) {
            log_1.log.error `Could not complete type generation for ${name}. Check debug logs`;
            log_1.log.debug `${error.err}`;
            log_1.log.debug `${stdout}`;
            log_1.log.debug `${error.stderr}`;
            info.declarator.typed = false;
        }
        else {
            log_1.log.debug `${name} was typed`;
            log_1.log.debug `${stdout}`;
            info.declarator.typed = true;
        }
        // Update package info file
        yield promises_1.default.writeFile(infoPath, JSON.stringify(info));
        return info.declarator.typed;
    });
}
exports.emitTypes = emitTypes;
//# sourceMappingURL=emitter.js.map