"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processPackage = void 0;
const path_1 = __importDefault(require("path"));
const defaults_1 = require("../config/defaults");
const api_1 = require("../npm/api");
const log_1 = require("../util/log");
const package_json_1 = require("../util/package-json");
const path_2 = require("../util/path");
const emitter_1 = require("./emitter");
const info_1 = require("./info");
/**
 * Starts the processing of a package
 *
 * @param {string} name Their name
 * @param {PackageConfig} config The tsconfig.json options record that
 *   should be used
 * @returns {Promise<boolean>} A promise that resolves true if the
 *   d.ts files were created successfully.
 */
function processPackage(name, config) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkgDir = (0, path_2.findPackage)(name);
        const packageInfo = yield (0, info_1.getPackageInfo)(name, pkgDir, config);
        if (packageInfo.declarator.typed) {
            log_1.log.debug `Package ${name} is already typed`;
            return false;
        }
        if (!config.ignoreDtCheck) {
            const existsTypes = yield (0, api_1.hasNpmPackage)(`@types/${name}`);
            if (existsTypes) {
                log_1.log.warn `An @types/${name} package is available!\n Install it at https://npmjs.com/package/@types/${name}. Use ignoreDtCheck to hide this warning.`;
                return false;
            }
        }
        const emitted = yield (0, emitter_1.emitTypes)(name, pkgDir, packageInfo);
        if (emitted) {
            yield (0, package_json_1.editPackageJson)(pkgDir, (atual) => {
                var _a, _b;
                const main = ((_a = atual.main) === null || _a === void 0 ? void 0 : _a.replace(/js$/, 'd.ts')) || 'index.d.ts';
                const outDir = ((_b = packageInfo.compilerOptions) === null || _b === void 0 ? void 0 : _b.outDir) ||
                    defaults_1.defaultOptions.defaults.compilerOptions.outDir;
                return Object.assign(Object.assign({}, atual), { types: path_1.default.join(outDir, main) });
            });
        }
        return emitted;
    });
}
exports.processPackage = processPackage;
//# sourceMappingURL=index.js.map