"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getPackageInfo = exports.infoVersion = exports.filename = void 0;
const promises_1 = __importDefault(require("fs/promises"));
const path_1 = __importDefault(require("path"));
const log_1 = require("../util/log");
/**
 * The filename to be our info and tsconfig
 */
exports.filename = 'declarator.generated.json';
/**
 * The package info compatibility version
 */
exports.infoVersion = 1;
/**
 * Attempts to read or create the package info located at root.
 *
 * @param name The package name
 * @param packagePath The path where the package is located
 * @param config The package tsconfig that should be used in case the
 *   package is not found or outdated
 * @returns A promise with the package info
 */
function getPackageInfo(name, packagePath, config) {
    return __awaiter(this, void 0, void 0, function* () {
        const infoPath = path_1.default.join(packagePath, exports.filename);
        let info;
        try {
            const json = (yield promises_1.default.readFile(infoPath)).toString();
            const content = JSON.parse(json);
            const { declarator } = content;
            const outdated = (declarator === null || declarator === void 0 ? void 0 : declarator.version) < exports.infoVersion;
            const newer = (declarator === null || declarator === void 0 ? void 0 : declarator.version) > exports.infoVersion;
            if (outdated || newer) {
                log_1.log.warn `package ${name} has ${outdated ? 'an outdated' : 'a newer'} declarator version of ${declarator === null || declarator === void 0 ? void 0 : declarator.version}`;
                yield promises_1.default.writeFile(path_1.default.join(packagePath, exports.filename + '.old'), 
                // Prettify it to be clear if a human tries to read it
                JSON.stringify(content, null, 2));
                log_1.log.info `A ${infoPath}.old was generated.`;
                // Skips to catch block
                throw null;
            }
            info = content;
        }
        catch (_a) {
            // Handles any case of error by overriding the package with a newly one.
            info = Object.assign({ declarator: { typed: false, version: exports.infoVersion, name } }, config);
            yield promises_1.default.writeFile(infoPath, JSON.stringify(info));
            log_1.log.debug `Wrote a default config for ${name}.`;
        }
        return info;
    });
}
exports.getPackageInfo = getPackageInfo;
//# sourceMappingURL=info.js.map